/*
 * Copyright (C) 2015 Paul Boddie <paul@boddie.org.uk>
 * Copyright (C) 2009 Qi Hardware Inc.
 * Authors: Xiangfu Liu <xiangfu@openmobilefree.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 3 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

/*
 * This file contains the configuration parameters for the NanoNote.
 */
#ifndef __NANONOTE_H__
#define __NANONOTE_H__

#include "jz4740_lcd.h"

/*
 * Display configuration
 */
#define LCD_BPP				LCD_COLOR32

/*
 * RAM configuration
 */
#define CONFIG_SYS_SDRAM_BASE		0x80000000

/*
 * SDRAM configuration (timings in ns)
 */
#define CONFIG_NR_DRAM_BANKS		1

#define SDRAM_BW16			1       /* Data bus width: 0-32bit, 1-16bit */
#define SDRAM_BANK4			1       /* Banks each chip: 0-2bank, 1-4bank */
#define SDRAM_ROW			13      /* Row address: 11 to 13 */
#define SDRAM_COL			9       /* Column address: 8 to 12 */
#define SDRAM_CASL			2       /* CAS latency: 2 or 3 */
#define SDRAM_TRAS			45      /* RAS# Active Time */
#define SDRAM_RCD			20      /* RAS# to CAS# Delay */
#define SDRAM_TPC			20      /* RAS# Precharge Time */
#define SDRAM_TRWL			7       /* Write Latency Time */
#define SDRAM_TREF			15625   /* Refresh period: 8192 cycles/64ms */

/*
 * Cache configuration
 */
#define CONFIG_SYS_DCACHE_SIZE		16384
#define CONFIG_SYS_ICACHE_SIZE		16384
#define CONFIG_SYS_CACHELINE_SIZE	32
#define KSEG0				0x80000000

/*
 * GPIO definition
 * See: http://en.qi-hardware.com/wiki/Hardware_basics
 */
#define GPIO_LCD_CS			(2 * 32 + 21)
#define GPIO_AMP_EN			(3 * 32 + 4)

#define GPIO_SDPW_EN			(3 * 32 + 2)
#define	GPIO_SD_DETECT			(3 * 32 + 0)

#define	GPIO_BUZZ_PWM			(3 * 32 + 27)
#define	GPIO_USB_DETECT			(3 * 32 + 28)

#define	GPIO_AUDIO_POP			(1 * 32 + 29)
#define GPIO_COB_TEST			(1 * 32 + 30)

#define	GPIO_KEYOUT_BASE		(2 * 32 + 10)
#define	GPIO_KEYIN_BASE			(3 * 32 + 18)
#define	GPIO_KEYIN_8			(3 * 32 + 26)

#define GPIO_SD_CD_N			GPIO_SD_DETECT	/* SD Card insert detect */
#define GPIO_SD_VCC_EN_N		GPIO_SDPW_EN	/* SD Card Power Enable */

#define GPIO_SD_CMD			(3 * 32 + 8)

#define SPEN				GPIO_LCD_CS	/* LCDCS :Serial command enable      */
#define SPDA				(2 * 32 + 22)	/* LCDSCL:Serial command clock input */
#define SPCK				(2 * 32 + 23)	/* LCDSDA:Serial command data input  */

#endif /* __NANONOTE_H__ */
